<?php

class BuybitApi
{
    const QiwiToBitcoin        = 1;
    const BitcoinToQiwi        = 2;
    const QiwiToBitcoinCash    = 3;
    const BitcoinCashToQiwi    = 4;
    const QiwiToLitecoin       = 5;
    const LitecoinToQiwi       = 6;
    const QiwiToEthereum       = 7;
    const EthereumToQiwi       = 8;
    const ExmoToQiwi           = 9;
    const QiwiToExmo           = 10;
    const SberbankToBitcoin    = 11;
    const MobileToBitcoin      = 12;
    const SberbankToLitecoin   = 13;
    const MobileToLitecoin     = 14;
    const SberbankToEthereum   = 15;
    const MobileToEthereum     = 16;
    const BitcoinToVisaMastercard = 17;
    const VtbToBitcoin         = 18;
    const AlfaBankToBitcoin    = 19;
    const TinkoffToBitcoin     = 20;
    const ExmoToVisaMastercard = 21;
    const PsBankToBitcoin      = 22;
    const OpenBankToBitcoin    = 23;
    const AvangardToBitcoin    = 24;
    const CardToBitcoin        = 25;
    const TinkoffToLitecoin    = 26;
    const TinkoffToEthereum    = 27;
    const AlfaBankToLitecoin   = 28;
    const AlfaBankToEthereum   = 29;

    protected $baseUrl = 'https://buybit.net/api/';

    protected $secret;

    public function __construct($secret)
    {
        $this->secret = $secret;
    }

    public function getStatus()
    {
        return $this->request('GET', 'status');
    }

    public function getMe()
    {
        return $this->request('GET', 'me');
    }

    public function getCurrencies()
    {
        return $this->request('GET', 'currencies');
    }

    public function getDirections()
    {
        return $this->request('GET', 'directions');
    }

    public function convert($directionId)
    {
        return $this->request('GET', 'convert/'. $directionId);
    }

    public function getOrder($orderId, $secret)
    {
        return $this->request('GET', 'order/'. $orderId .'/'. $secret);
    }

    public function cancelOrder($orderId, $secret)
    {
        return $this->request('DELETE', 'order/'. $orderId .'/'. $secret);
    }

    public function createOrder($directionId, $params)
    {
        return $this->request('POST', 'order/'. $directionId .'/create', $params);
    }

    protected function request($method, $url, array $params = [])
    {
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($ch, CURLOPT_USERAGENT, 'Buybit API Client');
        curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 30);
        curl_setopt($ch, CURLOPT_TIMEOUT, 30);
        curl_setopt($ch, CURLOPT_HTTPHEADER, [
            'Content-Type: application/json',
            'Accept: application/json',
            'Api-Secret: '. $this->secret,
        ]);

        if ($method == 'GET') {
            curl_setopt($ch, CURLOPT_URL, $this->baseUrl . $url .'?'. http_build_query($params));
            curl_setopt($ch, CURLOPT_HTTPGET, true);
        } else {
            curl_setopt($ch, CURLOPT_URL, $this->baseUrl . $url);
            curl_setopt($ch, CURLOPT_CUSTOMREQUEST, $method);
            curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($params));
        }

        $output = curl_exec($ch);
        $code   = (int) curl_getinfo($ch, CURLINFO_HTTP_CODE);
        $error  = curl_error($ch);

        curl_close($ch);

        if ($output) {
            $output = json_decode($output, true);

            if (json_last_error() === JSON_ERROR_NONE) {
                return new BuybitApiResponse($output, $code);
            }
        }

        if ($error) {
            throw new Exception($error);
        } else {
            throw new Exception('Unknown error.');
        }
    }

}

class BuybitApiResponse
{
    protected $output;
    protected $code;

    public function __construct($output, $code)
    {
        $this->output = $output;
        $this->code   = $code;
    }

    public function isOk()
    {
        return $this->code == 200;
    }

    public function isFail()
    {
        return $this->code != 200;
    }

    public function get()
    {
        return $this->output;
    }

}
